//. # React Collect
//.
//. [![Build Status](https://travis-ci.com/wearereasonablepeople/react-collect.svg?branch=master)](https://travis-ci.com/wearereasonablepeople/react-collect)
//.
//. Usage in Node depends on `--experimental-modules`.
//. With older Node versions, use [`esm`][1].

import React from 'react';

var Component = React.Component;
var createElement = React.createElement;
var Consumer = React.createContext ();

// identity :: a -> a
export function identity(x) {
  return x;
}

// getComponentName :: ReactComponent? -> String
export function getComponentName(Component) {
  return Component == null ?
         '<Null>' :
         (Component.displayName || Component.name || '<Anonymous>');
}

export function createBaseCollectionManager(item) {

  function BaseComponent(props) {
    Component.call (this, props);
    props.collect (item);
  }

  BaseComponent.prototype = Object.create (Component.prototype);

  BaseComponent.prototype.componentWillUnmount = function() {
    this.props.uncollect (item);
  };

  return BaseComponent;

}

//. ## API
//.
//# collect :: Any? -> ReactComponent -> ReactComponent
export function collect(item) {

  var BaseComponent = (item == null) ?
                      (Component) :
                      (createBaseCollectionManager (item));

  return function(UserComponent) {

    var name = getComponentName (UserComponent);

    function CollectorManager(props) {
      BaseComponent.call (this, props);
    }

    CollectorManager.prototype = Object.create (BaseComponent.prototype);

    CollectorManager.prototype.render = function() {
      return UserComponent && createElement (UserComponent, this.props);
    };

    CollectorManager.displayName = 'CollectorManager(' + name + ')';

    function Collecting(xs) {
      return createElement (Consumer, {}, function(ys) {
        return createElement (CollectorManager, Object.assign ({}, xs, ys));
      });
    }

    Collecting.displayName = 'Collecting(' + name + ')';

    return Collecting;

  };
}

//# Collector :: ReactComponent
//.
//. This component wraps your tree and ensures that any connected children will
//. be able to read from the store.
//.
//. Expects a single property `store` - the Redux store.
//.
//. Note that the item given to the store is automatically replaced after
//. the first render with the set of reducers associated with the currently
//. mounted connected components. Most of the time, you can just use `identity`
//. as the initial item.
//.
//. ```jsx
//. import {render} from 'react-dom';
//. import {Collector} from 'react-collect';
//. import {createStore} from 'redux';
//. import reduceReducers from 'reduce-reducers';
//. import App from './my-app';
//.
//. const store = createStore (x => x);
//. const onChange = rs => store.replaceReducer (reduceReducers (rs));
//.
//. render (
//.   <Collector onChange={onChange}><App /></Collector>,
//.   document.getElementById ('app')
//. );
//. ```
export function Collector(props) {
  if (typeof props.onChange !== 'function') {
    throw new TypeError ('Collector wants an onChange prop of type function');
  }
  Component.call (this, props);
  this.collection = new Set ();
}

Collector.prototype = Object.create (Component.prototype);

Collector.prototype.collect = function(item) {
  if (!this.collection.has (item)) {
    this.collection.add (item);
    this.forceUpdate ();
  }
};

Collector.prototype.uncollect = function(item) {
  if (this.collection.has (item)) {
    this.collection.delete (item);
    this.forceUpdate ();
  }
};

Collector.prototype.componentDidUpdate = function() {
  this.props.onChange (Array.from (this.collection));
};

Collector.prototype.componentWillUnmount = function() {
  if (this.collection.size > 0) {
    this.props.onChange ([]);
  }
};

Collector.prototype.render = function() {
  return createElement (
    Consumer.Provider,
    {value: {
      collect: this.collect.bind (this),
      uncollect: this.uncollect.bind (this)
    }},
    this.props.children
  );
};

//. [1]: https://github.com/standard-things/esm
